@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	7 January 2003
+ * Last Edited:	19 August 2003
  * 
  * The IMAP toolkit provided in this Distribution is
  * Copyright 1988-2003 University of Washington.
@@ -26,6 +26,8 @@
 				 * other:  initial text buffer size */
 #define MAXUSERFLAG 64		/* maximum length of a user flag */
 #define MAXAUTHENTICATORS 8	/* maximum number of SASL authenticators */
+				/* maximum number of messages */
+#define MAXMESSAGES (unsigned long) 100000000
 
 
 /* These can't be changed without changing code */
@@ -110,6 +112,10 @@
 #define SET_DEBUGSENSITIVE (long) 148
 #define GET_TCPDEBUG (long) 149
 #define SET_TCPDEBUG (long) 150
+#define GET_FREESTREAMSPAREP (long) 151
+#define SET_FREESTREAMSPAREP (long) 152
+#define GET_FREEBODYSPAREP (long) 153
+#define SET_FREEBODYSPAREP (long) 154
 
 	/* 2xx: environment */
 #define GET_USERNAME (long) 201
@@ -134,6 +140,8 @@
 #define SET_MAILSUBDIR 220
 #define GET_DISABLE822TZTEXT 221
 #define SET_DISABLE822TZTEXT 222
+#define GET_LIMITEDADVERTISE (long) 223
+#define SET_LIMITEDADVERTISE (long) 224
 	/* 3xx: TCP/IP */
 #define GET_OPENTIMEOUT (long) 300
 #define SET_OPENTIMEOUT (long) 301
@@ -163,6 +171,8 @@
 #define SET_SSLCERTIFICATEQUERY (long) 325
 #define GET_SSLFAILURE (long) 326
 #define SET_SSLFAILURE (long) 327
+#define GET_NEWSRCCANONHOST (long) 328
+#define SET_NEWSRCCANONHOST (long) 329
 
 	/* 4xx: network drivers */
 #define GET_MAXLOGINTRIALS (long) 400
@@ -211,6 +221,8 @@
 #define SET_IMAPTRYSSL (long) 443
 #define GET_FETCHLOOKAHEAD (long) 444
 #define SET_FETCHLOOKAHEAD (long) 445
+#define GET_NNTPRANGE (long) 446
+#define SET_NNTPRANGE (long) 447
 
 	/* 5xx: local file drivers */
 #define GET_MBXPROTECTION (long) 500
@@ -275,6 +287,10 @@
 #define SET_SASLUSESPTRNAME (long) 559
 #define GET_NETFSSTATBUG (long) 560
 #define SET_NETFSSTATBUG (long) 561
+#define GET_SNARFMAILBOXNAME (long) 562
+#define SET_SNARFMAILBOXNAME (long) 563
+#define GET_SNARFINTERVAL (long) 564
+#define SET_SNARFINTERVAL (long) 565
 
 /* Driver flags */
 
@@ -297,6 +313,8 @@
 #define DR_NONEWMAIL (long) 0x4000
 				/* driver does not announce new mail when RO */
 #define DR_NONEWMAILRONLY (long) 0x8000
+				/* driver can be halfopen */
+#define DR_HALFOPEN (long) 0x10000
 
 
 /* Cache management function codes */
@@ -558,6 +576,10 @@ ADDRESS {
   char *mailbox;		/* mailbox name */
   char *host;			/* domain name of mailbox's host */
   char *error;			/* error in address from SMTP module */
+  struct {
+    char *type;			/* address type (default "rfc822") */
+    char *addr;			/* address as xtext */
+  } orcpt;
   ADDRESS *next;		/* pointer to next address in list */
 };
 
@@ -565,8 +587,8 @@ ADDRESS {
 /* Message envelope */
 
 typedef struct mail_envelope {
-  unsigned int ngbogus : 1;	/* newsgroups may be bogus */
   unsigned int incomplete : 1;	/* envelope may be incomplete */
+  unsigned int imapenvonly : 1;	/* envelope only has IMAP envelope */
   char *remail;			/* remail header if any */
   ADDRESS *return_path;		/* error return address */
   char *date;			/* message composition date string */
@@ -655,6 +677,7 @@ BODY {
     unsigned long bytes;	/* size of text in octets */
   } size;
   char *md5;			/* MD5 checksum */
+  void *sparep;			/* spare pointer reserved for main program */
 };
 
 
@@ -980,6 +1003,11 @@ typedef struct mail_stream {
   ENVELOPE *env;		/* scratch buffer for envelope */
   BODY *body;			/* scratch buffer for body */
   SIZEDTEXT text;		/* scratch buffer for text */
+  struct {
+    char *name;			/* mailbox name to snarf from */
+    unsigned long time;		/* last snarf time */
+    long options;		/* snarf open options */
+  } snarf;
   union {			/* internal use only */
     struct {			/* search temporaries */
       STRINGLIST *string;	/* string(s) to search */
@@ -988,6 +1016,7 @@ typedef struct mail_stream {
     } search;
   } private;
 			/* reserved for use by main program */
+  void *sparep;			/* spare pointer */
   unsigned int spare : 1;	/* first spare bit */
   unsigned int spare2 : 1;	/* second spare bit */
   unsigned int spare3 : 1;	/* third spare bit */
@@ -1080,6 +1109,7 @@ typedef struct send_stream {
   long replycode;		/* last reply code */
   unsigned int debug : 1;	/* stream debug flag */
   unsigned int sensitive : 1;	/* sensitive data in progress */
+  unsigned int loser : 1;	/* server is a loser */
   union {			/* protocol specific */
     struct {			/* SMTP specific */
       unsigned int ok : 1;	/* supports ESMTP */
@@ -1112,6 +1142,7 @@ typedef struct send_stream {
 	  unsigned int success : 1;
 	} notify;
 	unsigned int full : 1;	/* return full headers */
+	char *envid;		/* envelope identifier as xtext */
       } dsn;
       struct {			/* size declaration */
 	unsigned int ok : 1;	/* supports SIZE */
@@ -1122,6 +1153,22 @@ typedef struct send_stream {
     } esmtp;
     struct {			/* NNTP specific */
       unsigned int post : 1;	/* supports POST */
+      struct {			/* NNTP extensions */
+	unsigned int ok : 1;	/* supports extensions */
+				/* supports LISTGROUP */
+	unsigned int listgroup : 1;
+	unsigned int over : 1;	/* supports OVER */
+	unsigned int hdr : 1;	/* supports HDR */
+	unsigned int pat : 1;	/* supports PAT */
+				/* supports STARTTLS */
+	unsigned int starttls : 1;
+				/* server has MULTIDOMAIN */
+	unsigned int multidomain : 1;
+				/* supports AUTHINFO USER */
+	unsigned int authuser : 1;
+				/* supported authenticators */
+	unsigned int sasl : MAXAUTHENTICATORS;
+      } ext;
     } nntp;
   } protocol;
 } SENDSTREAM;
@@ -1167,8 +1214,10 @@ typedef char *(*newsrcquery_t) (MAILSTREAM *stream,char *mulname,char *name);
 typedef char *(*userprompt_t) (void);
 typedef long (*append_t) (MAILSTREAM *stream,void *data,char **flags,
 			  char **date,STRING **message);
-typedef void (*freeenvelopesparep_t) (void **sparep);
 typedef void (*freeeltsparep_t) (void **sparep);
+typedef void (*freeenvelopesparep_t) (void **sparep);
+typedef void (*freebodysparep_t) (void **sparep);
+typedef void (*freestreamsparep_t) (void **sparep);
 typedef void *(*sslstart_t) (void *stream,char *host,unsigned long flags);
 typedef long (*sslcertificatequery_t) (char *reason,char *host,char *cert);
 typedef void (*sslfailure_t) (char *host,char *reason,unsigned long flags);
@@ -1425,6 +1474,7 @@ void *mail_parameters (MAILSTREAM *stream,long function,void *value);
 DRIVER *mail_valid (MAILSTREAM *stream,char *mailbox,char *purpose);
 DRIVER *mail_valid_net (char *name,DRIVER *drv,char *host,char *mailbox);
 long mail_valid_net_parse (char *name,NETMBX *mb);
+long mail_valid_net_parse_work (char *name,NETMBX *mb,char *service);
 void mail_scan (MAILSTREAM *stream,char *ref,char *pat,char *contents);
 void mail_list (MAILSTREAM *stream,char *ref,char *pat);
 void mail_lsub (MAILSTREAM *stream,char *ref,char *pat);
@@ -1513,7 +1563,8 @@ long mail_search_text (MAILSTREAM *stream,unsigned long msgno,char *section,
 long mail_search_body (MAILSTREAM *stream,unsigned long msgno,BODY *body,
 		       char *prefix,unsigned long section,long flags);
 long mail_search_string (SIZEDTEXT *s,char *charset,STRINGLIST **st);
-long mail_search_keyword (MAILSTREAM *stream,MESSAGECACHE *elt,STRINGLIST *st);
+long mail_search_keyword (MAILSTREAM *stream,MESSAGECACHE *elt,STRINGLIST *st,
+			  long flag);
 long mail_search_addr (ADDRESS *adr,STRINGLIST *st);
 char *mail_search_gets (readfn_t f,void *stream,unsigned long size,
 			GETS_DATA *md);
@@ -1530,7 +1581,8 @@ unsigned long *mail_sort_msgs (MAILSTREAM *stream,char *charset,SEARCHPGM *spg,
 			       SORTPGM *pgm,long flags);
 SORTCACHE **mail_sort_loadcache (MAILSTREAM *stream,SORTPGM *pgm);
 unsigned int mail_strip_subject (char *t,char **ret);
-unsigned int mail_strip_subject_aux (char *t,char **ret);
+char *mail_strip_subject_wsp (char *s);
+char *mail_strip_subject_blob (char *s);
 int mail_sort_compare (const void *a1,const void *a2);
 unsigned long mail_longdate (MESSAGECACHE *elt);
 THREADNODE *mail_thread (MAILSTREAM *stream,char *type,char *charset,
