@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	25 February 2002
+ * Last Edited:	16 June 2003
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2002 University of Washington.
+ * Copyright 1988-2003 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -41,8 +41,6 @@ static char *sharedHome = NIL;	/* shared home directory */
 static short anonymous = NIL;	/* is anonymous */
 static short restrictBox = NIL;	/* is a restricted box */
 static short has_no_life = NIL;	/* is a cretin with no life */
-				/* warning on EACCES errors on .lock files */
-static short lockEaccesError = T;
 static short hideDotFiles = NIL;/* hide files whose names start with . */
 				/* advertise filesystem root */
 static short advertisetheworld = NIL;
@@ -249,11 +247,6 @@ void *env_parameters (long function,void *value)
   case GET_LOCKTIMEOUT:
     ret = (void *) locktimeout;
     break;
-  case SET_LOCKEACCESERROR:
-    lockEaccesError = value ? T : NIL;
-  case GET_LOCKEACCESERROR:
-    ret = (void *) (lockEaccesError ? VOIDT : NIL);
-    break;
   case SET_HIDEDOTFILES:
     hideDotFiles = value ? T : NIL;
   case GET_HIDEDOTFILES:
@@ -466,21 +459,30 @@ static struct passwd *pwuser (char *user)
  *	    argument vector
  * Returns: password entry if validated
  *
- * Tries all-lowercase form of user name if given user name fails
  * Tries password+1 if password fails and starts with space
  */
 
 static struct passwd *valpwd (char *user,char *pwd,int argc,char *argv[])
 {
+  char *s;
   struct passwd *pw;
-  char *usr;
-  if (pw = pwuser (user)) {	/* can get user? */
-    usr = cpystr (pw->pw_name);	/* copy returned name in case we need it */
-    if (!(pw = checkpw (pw,pwd,argc,argv)) && (*pwd == ' ') &&
-	(pw = pwuser (usr))) pw = checkpw (pw,pwd+1,argc,argv);
-    fs_give ((void **) &usr);	/* don't need copy of name any more */
+  struct passwd *ret = NIL;
+  if (auth_md5.server) {	/* using CRAM-MD5 authentication? */
+    if (s = auth_md5_pwd (user)) {
+      if (!strcmp (s,pwd) || ((*pwd == ' ') && pwd[1] && !strcmp (s,pwd+1)))
+	ret = pwuser (user);	/* validated, get passwd entry for user */
+      memset (s,0,strlen (s));	/* erase sensitive information */
+      fs_give ((void **) &s);
+    }
   }
-  return pw;
+  else if (pw = pwuser (user)) {/* can get user? */
+    s = cpystr (pw->pw_name);	/* copy returned name in case we need it */
+    if (*pwd && !(ret = checkpw (pw,pwd,argc,argv)) &&
+	(*pwd == ' ') && pwd[1] && (ret = pwuser (s)))
+      ret = checkpw (pw,pwd+1,argc,argv);
+    fs_give ((void **) &s);	/* don't need copy of name any more */
+  }
+  return ret;
 }
 
 /* Server log in
@@ -494,7 +496,6 @@ static struct passwd *valpwd (char *user,char *pwd,int argc,char *argv[])
 
 long server_login (char *user,char *pwd,char *authuser,int argc,char *argv[])
 {
-  char *s;
   struct passwd *pw = NIL;
   int level = LOG_NOTICE;
   char *err = "failed";
@@ -507,16 +508,6 @@ long server_login (char *user,char *pwd,char *authuser,int argc,char *argv[])
   }
   else if (logtry-- <= 0) err = "excessive login failures";
   else if (disablePlaintext) err = "disabled";
-  else if (auth_md5.server) {	/* using CRAM-MD5 authentication? */
-    if (s = (authuser && *authuser) ?
-	auth_md5_pwd (authuser) : auth_md5_pwd (user)) {
-      if (!strcmp (s,pwd) || ((*pwd == ' ') && !strcmp (s,pwd+1)))
-	pw = pwuser (user);	/* CRAM-MD5 authentication validated */
-      memset (s,0,strlen (s));	/* erase sensitive information */
-      fs_give ((void **) &s);
-    }
-    else err = "failed: no CRAM-MD5 entry";
-  }
   else if (!(authuser && *authuser)) pw = valpwd (user,pwd,argc,argv);
   else if (valpwd (authuser,pwd,argc,argv)) pw = pwuser (user);
   if (pw && pw_login (pw,authuser,pw->pw_name,NIL,argc,argv)) return T;
@@ -901,6 +892,7 @@ long dotlock_lock (char *file,DOTLOCK *base,int fd)
       mm_log (tmp,WARN);
     }
     mask = umask (0);
+    unlink (base->lock);	/* try to remove the old file */
 				/* seize the lock */
     if ((i = open (base->lock,O_WRONLY|O_CREAT,(int) lock_protection)) >= 0) {
       close (i);		/* don't need descriptor any more */
@@ -952,13 +944,20 @@ long dotlock_lock (char *file,DOTLOCK *base,int fd)
       }
       close (pi[0]); close (pi[1]);
     }
-    if (lockEaccesError) {	/* punt silently if paranoid site */
-      sprintf (tmp,"Mailbox vulnerable - directory %.80s",base->lock);
-      if (s = strrchr (tmp,'/')) *s = '\0';
-      strcat (tmp," must have 1777 protection");
-      mm_log (tmp,WARN);
+				/* find directory/file delimiter */
+    if (s = strrchr (base->lock,'/')) {
+      *s = '\0';		/* tie off at directory */
+      sprintf(tmp,		/* generate default message */
+	      "Mailbox vulnerable - directory %.80s must have 1777 protection",
+	      base->lock);
+				/* definitely not 1777 if can't stat */
+      mask = stat (base->lock,&sb) ? 0 : (sb.st_mode & 1777);
+      *s = '/';			/* restore lock name */
+      if (mask != 1777) {	/* default warning if not 1777 */
+	MM_LOG (tmp,WARN);
+	break;
+      }
     }
-    break;
   default:
     sprintf (tmp,"Mailbox vulnerable - error creating %.80s: %s",
 	     base->lock,strerror (errno));
