@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: basic.c,v 4.35 2002/12/03 06:31:23 jpf Exp $";
+static char rcsid[] = "$Id: basic.c,v 4.37 2003/03/11 03:27:05 jpf Exp $";
 #endif
 /*
  * Program:	Cursor manipulation functions
@@ -286,13 +286,16 @@ int f, n;	/* default Flag & Numeric argument */
 	 * PLUS: if there's a quote string, a quoted-to-non-quoted
 	 *	 line transition.
 	 */
-	quoted = glo_quote_str
-	  ? quote_match(glo_quote_str, curwp->w_dotp, qstr, NLINE) : 0;
+	quoted = (glo_quote_str || (Pmaster && Pmaster->quote_str))
+	  ? quote_match(glo_quote_str ? glo_quote_str : Pmaster->quote_str,
+			curwp->w_dotp, qstr, NLINE) : 0;
 	qlen   = quoted ? strlen(qstr) : 0;
 	while(lback(curwp->w_dotp) != curbp->b_linep
 	      && llength(lback(curwp->w_dotp)) > qlen
-	      && (glo_quote_str
-		  ? (quoted == quote_match(glo_quote_str, lback(curwp->w_dotp),
+	      && ((glo_quote_str || (Pmaster && Pmaster->quote_str))
+		  ? (quoted == quote_match(glo_quote_str ? glo_quote_str
+					   : Pmaster->quote_str, 
+					   lback(curwp->w_dotp),
 					   qstr2, NLINE)
 		     && !strcmp(qstr, qstr2))
 		  : 1)
@@ -356,14 +359,17 @@ int f, n;	/* default Flag & Numeric argument */
 	 * PLUS: if there's a quote string, a quoted-to-non-quoted
 	 *	 line transition.
 	 */
-	quoted = glo_quote_str
-	  ? quote_match(glo_quote_str, curwp->w_dotp, qstr, NLINE) : 0;
+	quoted = ((glo_quote_str || (Pmaster && Pmaster->quote_str))
+	  ? quote_match(glo_quote_str ? glo_quote_str : Pmaster->quote_str,
+			curwp->w_dotp, qstr, NLINE) : 0);
 	qlen   = quoted ? strlen(qstr) : 0;
 	
 	while(curwp->w_dotp != curbp->b_linep
 	      && llength(lforw(curwp->w_dotp)) > qlen
-	      && (glo_quote_str
-		  ? (quoted == quote_match(glo_quote_str, lforw(curwp->w_dotp),
+	      && ((glo_quote_str || (Pmaster && Pmaster->quote_str))
+		  ? (quoted == quote_match(glo_quote_str ? glo_quote_str
+					   : Pmaster->quote_str,
+					   lforw(curwp->w_dotp),
 					   qstr2, NLINE)
 		     && !strcmp(qstr, qstr2))
 		  : 1)
