@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.79 2002/12/04 01:15:07 hubert Exp $";
+static char rcsid[] = "$Id: file.c,v 4.81 2003/03/11 03:27:05 jpf Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -376,7 +376,9 @@ insmsgchar(c)
 	char *p;
 
 	lnewline();
-	for(p = glo_quote_str; p && *p; p++)
+	for(p = (glo_quote_str ? glo_quote_str
+		 : (Pmaster ? Pmaster->quote_str : NULL));
+	    p && *p; p++)
 	  if(!linsert(1, *p))
 	    return(0);
     }
