@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: line.c,v 4.28 2002/12/12 07:13:42 jpf Exp $";
+static char rcsid[] = "$Id: line.c,v 4.30 2003/03/11 03:27:05 jpf Exp $";
 #endif
 /*
  * Program:	Line management routines
@@ -633,7 +633,9 @@ lisblank(line)
     int n = 0;
     char qstr[NLINE];
 
-    n = (glo_quote_str && quote_match(glo_quote_str,line,qstr,NLINE))
+    n = ((glo_quote_str || (Pmaster && Pmaster->quote_str))
+	 && quote_match(glo_quote_str ? glo_quote_str : Pmaster->quote_str,
+			line, qstr, NLINE))
 	  ? strlen(qstr) : 0;
 
     for(; n < llength(line); n++)
